;;; Test for shared writes to a single hash entry for ABCL
;;; 
;;; Try to abstract the problems with
;;; https://lsw2.googlecode.com/svn/branches/bona/util/jargrep.lisp
;;; into a "just athreaded access to the hashtable problem."
;;;
;;; dc:author "Mark <evenson@panix.com>" ;
;;; dc:date  27-SEP-2013

#|
Usage:

(run 2) 

(run 1000) 

|#

(defparameter *shared* (make-hash-table :test 'equal))

(defun sleep-random-interval ()
  (sleep (/ (random 1000) 1000)))
  
(defun increment-squares (basis count)
  "Store COUNT entries in the *shared* hashtable at the keys given by the sequence  (BASIS (* BASIS BASIS) (* (* BASIS BASIS) BASIS) …"
  (loop 
     :for i :below count
     :for index = basis then (* index basis)
     :doing (progn 
	      (sleep-random-interval)
	      (setf (gethash index *shared*)
		    (get-universal-time))))
  count)

(defun run (count)
  (let ((primes '(2 3 5 7 11 13 15 17)))
    (clrhash *shared*)
    (format t "~&Spawning ~A threads..." (length primes))
    (format t 
	    " Done.~%Spawned threads that should yield ~A entries, and the shared value is ~A.~%"
	    (loop :for thread :in 
	       (loop :for n :in primes
		  :collecting (threads:make-thread 
			       (lambda () (increment-squares n count))))
	       :summing (threads:thread-join thread))
	    (hash-table-count *shared*))))
	       
